#!/usr/bin/python

import gtk, gobject, re

#http://developer.gnome.org/pygtk/2.22/class-gtkicontheme.html#method-gtkicontheme--list-contexts

class IcoBrowse(gtk.Dialog):
	def __init__(self, message="", default_text='', modal=True):
		gtk.Dialog.__init__(self)
		self.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CLOSE,
		      gtk.STOCK_OK, gtk.RESPONSE_ACCEPT)
		#self.set_title("Icon search")
		if modal:
			self.set_modal(True)
		self.set_border_width(5)
		self.set_size_request(400, 300)
		self.combobox=gtk.combo_box_new_text()
		self.combobox.set_size_request(200, 20)
		hbox=gtk.HBox(False,2)
		
		#format: actual icon, name, context
		self.model=gtk.ListStore(gtk.gdk.Pixbuf, gobject.TYPE_STRING, gobject.TYPE_STRING)
		self.modelfilter=self.model.filter_new()
		self.modelfilter.set_visible_func(self.search_icons)
		self.iconview=gtk.IconView()
		self.iconview.set_model(self.modelfilter)
		self.iconview.set_pixbuf_column(0)
		self.iconview.set_text_column(1)
		self.iconview.set_selection_mode(gtk.SELECTION_SINGLE)
		self.iconview.set_item_width(72)
		self.iconview.set_size_request(200, 220)
		defaulttheme=gtk.icon_theme_get_default()
		self.combobox.connect('changed', self.category_changed)
		self.refine=gtk.Entry()
		self.refine.connect('changed', self.category_changed)
		self.refine.set_icon_from_stock(gtk.ENTRY_ICON_SECONDARY, gtk.STOCK_FIND)
		self.refine.set_size_request(200, 30)
		list2=[]
		for c in defaulttheme.list_contexts():
			current=defaulttheme.list_icons(context=c)
			list2+=set(current)
			self.combobox.append_text(c)
			for i in current:
				self.model.append([defaulttheme.load_icon(i, 32,
									  gtk.ICON_LOOKUP_USE_BUILTIN),
									  i,c])
		other=list(set(defaulttheme.list_icons())-(set(list2)))
		for i in other:
			self.model.append([defaulttheme.load_icon(i, 32,
									  gtk.ICON_LOOKUP_USE_BUILTIN),
									  i,"Other"])
		self.combobox.prepend_text("Other")
		scrolled = gtk.ScrolledWindow()
		scrolled.add(self.iconview)
		scrolled.props.hscrollbar_policy = gtk.POLICY_NEVER
		scrolled.props.vscrollbar_policy = gtk.POLICY_AUTOMATIC
		hbox.add(self.combobox)
		hbox.add(self.refine)
		self.vbox.add(hbox)
		self.vbox.add(scrolled)
		self.combobox.set_active(0)
		
		self.iconview.connect('selection-changed', self.get_icon_name)
		
		self.vbox.show_all()

	def set_defaults(self, icon_name):
		if icon_name != "" and icon_name != None:
			cmodel = self.combobox.get_model()
			for i in xrange(len(self.model)):
				if self.model[i][1] == icon_name:
					self.refine.props.text=icon_name
					for j in xrange(len(cmodel)):
						if self.model[i][2] == cmodel[j][0]:
							print cmodel[j]
							self.combobox.set_active(j)
							break
					break

	def get_icon_name(self, widget):
		path=self.iconview.get_selected_items()
		if len(path) > 0:
			return self.modelfilter[path[0]][1]

	def category_changed(self, widget):
		self.modelfilter.refilter()

	def search_icons(self, tree, iter):
		search_term = self.combobox.get_active_text()
		search_term2 = self.refine.props.text
		if self.model.get_value(iter, 2) == search_term:
			if search_term2 == None or search_term2 == "" or re.search(search_term2, self.model.get_value(iter,1)):
				return True
			return False
		else:
			return False



import gtk
import os
import glib
from xdg import BaseDirectory
import re #This is to autoset file mode for *.desktop icons
import ConfigParser
from lxml import etree
from cb_itemtypes import *

#test lines:
#import cbmenu,cb_itemtypes
#import gtk
#blah=cbmenu.MenuFile("/home/shadowkyogre/.config/compiz/boxmenu/menu.xml")
#blurg=gtk.Window()
#blurg.add(blah)
#blurg.show_all()

class MenuFile(gtk.ScrolledWindow):

	def __init__(self,filename):
		gtk.ScrolledWindow.__init__(self)

		self.model = gtk.TreeStore(object)
		self.add_menu_file(filename)
		self.filename=filename

		self.props.hscrollbar_policy = gtk.POLICY_AUTOMATIC #because you really might want to read some of the stuff hanging off
		self.props.vscrollbar_policy = gtk.POLICY_AUTOMATIC
		self.treeview = gtk.TreeView(self.model)
		self.treeview.set_reorderable(True)
		cell = gtk.CellRendererText()
		elements = gtk.TreeViewColumn('Item', cell)
		elements.set_cell_data_func(cell, self.get_type)
		self.treeview.append_column(elements)

		name = gtk.TreeViewColumn('Name')

		cell = gtk.CellRendererPixbuf()
		name.pack_start(cell, False)
		name.set_cell_data_func(cell, self.get_icon)

		cell = gtk.CellRendererText()
		name.pack_start(cell)
		name.set_cell_data_func(cell, self.get_name)

		self.treeview.append_column(name)
		self.add(self.treeview)
		targets = [
			('deskmenu-element', gtk.TARGET_SAME_WIDGET, 0),
			#('deskmenu-element', gtk.TARGET_SAME_APP, 0),
			#('deskmenu-element', gtk.TREE_VIEW_ITEM, 0),
			('text/uri-list', 0, 1),
		]
		self.treeview.enable_model_drag_source(gtk.gdk.BUTTON1_MASK, targets, gtk.gdk.ACTION_DEFAULT|gtk.gdk.ACTION_MOVE)
		self.treeview.enable_model_drag_dest(targets, gtk.gdk.ACTION_MOVE)

		self.treeview.connect('drag-data-get', self.on_drag_data_get)
		self.treeview.connect('drag-data-received', self.on_drag_data_received)

		self.treeview.connect('row-activated', self.on_row_activated)

		self.treeview.connect('button-press-event', self.on_treeview_button_press_event)
		self.treeview.expand_all()

		self.selection = self.treeview.get_selection()
		self.selection.connect('changed', self.on_selection_changed)

		self.popup = gtk.Menu()
		self.edit_menu = gtk.ImageMenuItem(stock_id=gtk.STOCK_EDIT)
		self.edit_menu.connect('activate', self.on_edit_clicked)
		self.popup.append(self.edit_menu)
		self.delete_menu = gtk.ImageMenuItem(stock_id=gtk.STOCK_DELETE)
		self.delete_menu.connect('activate', self.on_delete_clicked)
		self.popup.append(self.delete_menu)
		self.popup.show_all()

		self.show_all()

	def add_menu_file(self, filename):
		self.menufile = etree.parse(filename)
		self.menu = Menu(self.menufile.getroot())
		self.add_menu(self.menu)

	def add_menu(self, m, parent=None):
		for item in m.children:
			iter = self.model.append(parent, [item])
			if item.node.tag == 'menu':
				self.add_menu(item, iter)

	def get_name(self, column, cell, model, iter):
		name = model.get_value(iter, 0).get_name()
		if name is None:
			name = ''
		cell.set_property('text', name)

	def get_type(self, column, cell, model, iter):
		typ = model.get_value(iter, 0).get_type()
		if typ is None:
			typ = ''
		cell.set_property('text', typ)

	def get_icon(self, column, cell, model, iter):
		icon = model.get_value(iter, 0).get_icon()
		icon_mode = model.get_value(iter, 0).get_icon_mode()
		#somehow does not set icon until selection change with things that don't have icons originally!
		if icon is not None:
			if icon_mode is not None:
				w = gtk.icon_size_lookup(gtk.ICON_SIZE_MENU)
				try:
					cell.set_property('pixbuf', gtk.gdk.pixbuf_new_from_file_at_size(os.path.expanduser(icon), w[0], w[0]))
					cell.set_property('icon-name', None) #possibly reduntant safety measure
				except glib.GError:
					cell.set_property('icon-name', None)
					cell.set_property('pixbuf', None)
			else:
				cell.set_property('icon-name', icon)
				cell.set_property('pixbuf', None) #possibly reduntant safety measure
		else:
			cell.set_property('icon-name', None)
			cell.set_property('pixbuf', None)

	def on_edit_clicked(self, widget):
		if hasattr(self, 'currently_editing'):
			self.currently_editing.destroy()
		model=self.selection.get_selected()[0]
		iter=self.selection.get_selected()[1]
		self.currently_editing=EditItemPanel(model,iter)

	def on_delete_clicked(self, widget):

		model, row = self.selection.get_selected()

		parent = None
		if row:
			current = model[row][0].node

			if current.tag == 'menu' and len(current):
				warning = gtk.MessageDialog(self.get_toplevel(), \
				gtk.DIALOG_MODAL, gtk.MESSAGE_INFO, gtk.BUTTONS_NONE, \
				'Delete menu element with %s children?' %len(current))
				warning.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_DELETE, gtk.RESPONSE_ACCEPT)
				if warning.run() != gtk.RESPONSE_ACCEPT:
					warning.destroy()
					return
				warning.destroy()

			parent = model[row].parent
			if parent is not None:
				parent = parent[0].node
			else:
				parent = self.menu.node
			parent.remove(current)
			model.remove(row)

		#write_menu()

	def on_close_clicked(self, widget):

		self.write_menu()

	def on_drag_data_get(self, treeview, context, selection, target_id,
						   etime):
		treeselection = treeview.get_selection()
		model, iter = treeselection.get_selected()
		data = model.get_string_from_iter(iter)

		selection.set(selection.target, 8, data)

	def on_drag_data_received(self, treeview, context, x, y, selection,
								info, etime):
		model = treeview.get_model()
		data = selection.data

		drop_info = treeview.get_dest_row_at_pos(x, y)
		if selection.type == 'deskmenu-element':
			source = model[data][0]
			if drop_info:
				path, position = drop_info
				siter = model.get_iter(data)
				diter = model.get_iter(path)

				if model.get_path(model.get_iter_from_string(data)) == path:
					return

				dest = model[path][0]
				if context.action == gtk.gdk.ACTION_MOVE:
					source.node.getparent().remove(source.node)

				if dest.node.tag == 'menu' and position in (gtk.TREE_VIEW_DROP_INTO_OR_BEFORE,
					gtk.TREE_VIEW_DROP_INTO_OR_AFTER):
					dest.node.append(source.node)
					fiter = model.append(diter, row=(source,))
				else:
					i = dest.node.getparent().index(dest.node)
					if position in (gtk.TREE_VIEW_DROP_INTO_OR_BEFORE,
						gtk.TREE_VIEW_DROP_BEFORE):
						dest.node.getparent().insert(i, source.node)
						fiter = model.insert_before(None, diter, row=(source,))
					else:
						dest.node.getparent().insert(i+1, source.node)
						fiter = model.insert_after(None, diter, row=(source,))

				if model.iter_has_child(siter):
					citer = model.iter_children(siter)
					while citer is not None:
						model.append(fiter, row=(model[citer][0],))
						citer = model.iter_next(citer)
				if context.action == gtk.gdk.ACTION_MOVE:
					context.finish(True, True, etime)

		elif selection.type == 'text/uri-list':
			print selection.data
			if drop_info:
				path, position = drop_info
				uri = selection.data.replace('file:///', '/').replace("%20"," ").strip()
				entry = ConfigParser.ConfigParser()
				entry.read(uri)
				launcher = Launcher()
				launcher.name = etree.SubElement(launcher.node, 'name')
				launcher.icon = etree.SubElement(launcher.node, 'icon')
				launcher.command = etree.SubElement(launcher.node, 'command')
				try:
					launcher.name.text = entry.get('Desktop Entry', 'Name')
					if re.search("/", entry.get('Desktop Entry', 'Icon')):
						launcher.icon.attrib['mode1'] = 'file'
					launcher.icon.text = entry.get('Desktop Entry', 'Icon')
					launcher.command.text = entry.get('Desktop Entry', 'Exec').split('%')[0]
				except ConfigParser.Error:
					return
				dest = model[path][0]
				diter = model.get_iter(path)
				if dest.node.tag == 'menu' and position in (gtk.TREE_VIEW_DROP_INTO_OR_BEFORE,
					gtk.TREE_VIEW_DROP_INTO_OR_AFTER):
					dest.node.append(launcher.node)
					fiter = model.append(diter, row=(launcher,))
				else:
					i = dest.node.getparent().index(dest.node)
					if position in (gtk.TREE_VIEW_DROP_INTO_OR_BEFORE,
						gtk.TREE_VIEW_DROP_BEFORE):
						dest.node.getparent().insert(i, launcher.node)
						fiter = model.insert_before(None, diter, row=(launcher,))
					else:
						dest.node.getparent().insert(i+1, launcher.node)
						fiter = model.insert_after(None, diter, row=(launcher,))
				if context.action == gtk.gdk.ACTION_MOVE:
					context.finish(True, True, etime)

		return

	def on_selection_changed(self, selection):

		model, row = selection.get_selected()

		sensitive = row and model.get_value(row, 0).editable

		self.edit_menu.props.sensitive = sensitive
		self.delete_menu.props.sensitive = row

	def on_row_activated(self, treeview, path, view_column):

		model = treeview.get_model()
		if hasattr(self, 'currently_editing'):
			self.currently_editing.destroy()
		self.currently_editing=EditItemPanel(model, model.get_iter(path))
		#change this to actually fill out item panel

	def on_treeview_button_press_event(self, treeview, event):
		if event.button == 3:
			pthinfo = self.treeview.get_path_at_pos(int(event.x), int(event.y))
			if pthinfo is not None:
				path, col, cellx, celly = pthinfo
				treeview.grab_focus()
				treeview.set_cursor(path, col, 0)
				self.popup.popup(None, None, None, event.button, event.time)
			return 1

	def get_icon_mode(self):
		iconnode = self.node.find('icon')
		if iconnode is not None:
			if iconnode.attrib.get('mode1') == 'file':
				return iconnode.attrib.get('mode1')
		else:
			return None

	def indent(self,elem, level=0):
		i = "\n" + level*"\t" #used to be "  ", use actual tabs
		if len(elem):
			if not elem.text or not elem.text.strip():
				elem.text = i + "\t"
			for e in elem:
				self.indent(e, level+1)
				if not e.tail or not e.tail.strip():
					e.tail = i + "\t"
			if not e.tail or not e.tail.strip():
				e.tail = i
		else:
			if level and (not elem.tail or not elem.tail.strip()):
				elem.tail = i

	def write_menu(self):
		self.indent(self.menu.node)
		self.menufile.write(open(self.filename, 'w'))

class EditItemPanel(gtk.HBox):

	def __init__(self, model=None, row=None, element=None):
		gtk.HBox.__init__(self)
		self.vbox_image_grid=gtk.VBox(True, 2)
		self.vbox_other_options=gtk.VBox(True, 2)

		self.add(self.vbox_image_grid)
		self.add(self.vbox_other_options)

		self.props.spacing = 6

		if element is None:
			if not row:
				return
			element = model.get_value(row, 0)

		if not element.editable:
			return

		icons,widgets=element.get_options()

		for icon in icons:
			self.vbox_image_grid.add(icon)

		for widget in widgets:
			self.vbox_other_options.add(widget)

		self.show_all()


#!/usr/bin/env python

import compizconfig

context = compizconfig.Context()

if 'run_command0_key' in context.Plugins['core'].Display:

    print "setting up for compiz 0.7.x"

    context.Plugins['core'].Display['command0'].Value = 'compiz-deskmenu'
    context.Plugins['core'].Display['run_command0_key'].Value = '<Control>space'

    if 'vpswitch' in context.Plugins:
        vpswitch = context.Plugins['vpswitch']
        if not vpswitch.Enabled:
            vpswitch.Enabled = True
        vpswitch.Display['init_plugin'].Value = 'core'
        vpswitch.Display['init_action'].Value = 'run_command0_key'

    context.Write()

else:
    
    print "compiz-deskmenu requires compiz 0.7.x for full functionality"



import gtk
from lxml import etree
from cbutil import *

class Item(object):

	def __init__(self, node=None, parent=None, type=None):

		self.editable = False

		if node is None:
			self.node = etree.Element('item')
			if type is not None:
				self.node.attrib['type'] = type

		else:
			self.node = node

	def get_name(self):
		return None

	def get_type(self):
		return 'Item'

	def get_icon(self):
		return None

	def get_icon_mode(self):
		return None

class Launcher(Item):

	def __init__(self, node=None):

		Item.__init__(self, node)

		if node is None:
			self.node = etree.Element('item', type='launcher')

		self.editable = True

	def get_name(self):

		subnode = self.node.find('name')
		if subnode is not None:
			name = subnode.text
			if subnode.attrib.get('mode') == 'exec':
				name = 'exec: %s' %name
			return name
		else:
			return None

	def get_type(self):
		return 'Launcher'

	def get_icon(self):
		iconnode = self.node.find('icon')
		if iconnode is not None:
			return iconnode.text
		else:
			return None

	def get_icon_mode(self):
		iconnode = self.node.find('icon')
		if iconnode is not None:
			if iconnode.attrib.get('mode1') == 'file':
				return iconnode.attrib.get('mode1')
		else:
			return None

	def get_options(self):

		retlist = []
		icons = []

		namenode = self.node.find('name')
		if namenode is not None:
			name = namenode.text
			if namenode.attrib.get('mode') == 'exec':
				name_mode="Execute"
			else:
				name_mode="Normal"
		else:
			name = ''
			name_mode="Normal"

		widget = CommandText(mode=name_mode,text=name)
		widget.connect('text-changed', self.on_subnode_changed, 'name')
		widget.connect('mode-changed', self.on_name_mode_changed)

		retlist.append(widget)

		iconnode = self.node.find('icon')
		if iconnode is not None:
			icon = iconnode.text
			if iconnode.attrib.get('mode1') == 'file':
				icon_mode = "File path"
			else:
				icon_mode="Normal"
		else:
			icon = ''
			icon_mode="Normal"
		widget = IconSelector(mode=icon_mode,text=icon)

		widget.connect('text-changed', self.on_subnode_changed, 'icon')
		widget.connect('mode-changed', self.on_icon_mode_changed)

		icons.append(widget)

		commandnode = self.node.find('command')
		if commandnode is not None:
			command = commandnode.text
			if commandnode.attrib.get('mode2') == 'pipe':
				command_mode="Pipe"
			else:
				command_mode="Normal"
		else:
			command = ''
			command_mode = "Normal"
		widget = CommandText(label_text="Command",mode=command_mode,text=command, alternate_mode="Pipe")

		widget.connect('text-changed', self.on_subnode_changed, 'command')
		widget.connect('mode-changed', self.on_command_mode_changed)

		retlist.append(widget)

		widget = gtk.CheckButton("Cache output of pipe")
		widget.props.active = commandnode is not None and commandnode.attrib.get('mode2') == 'pipe' and commandnode.attrib.get('cache') == 'true'
		widget.connect('toggled', self.on_cached_changed)
		retlist.append(widget)

		return icons,retlist

	def on_subnode_changed(self, widget, text, tag):
		subnode = self.node.find(tag)
		if text:
			if subnode is None:
				subnode = etree.SubElement(self.node, tag)
			subnode.text = text
		else:
			if subnode is not None:
				self.node.remove(subnode)

	def on_name_mode_changed(self, widget, text):
		namenode = self.node.find('name')
		print text
		if text == "Execute":
			if namenode is None:
				namenode = etree.SubElement(self.node, 'name')
			namenode.attrib['mode'] = 'exec'
		elif 'mode' in namenode.attrib:
			del namenode.attrib['mode']

	def on_icon_mode_changed(self, widget, text):
		iconnode = self.node.find('icon')
		if text == "File path":
			if iconnode is None:
				iconnode = etree.SubElement(self.node, 'icon')
			iconnode.attrib['mode1'] = 'file'
		elif 'mode1' in iconnode.attrib:
			del iconnode.attrib['mode1']

	def on_command_mode_changed(self, widget, text):
		commandnode = self.node.find('command')
		if text == "Pipe":
			if commandnode is None:
				commandnode = etree.SubElement(self.node, 'command')
			commandnode.attrib['mode2'] = 'pipe'
		elif 'mode2' in commandnode.attrib:
			del commandnode.attrib['mode2']

	def on_cached_changed(self, widget):
		commandnode = self.node.find('command')
		if widget.props.active:
			if commandnode is None:
				commandnode = etree.SubElement(self.node, 'command')
			commandnode.attrib['cache'] = 'true'
		elif 'cache' in commandnode.attrib:
			del commandnode.attrib['cache']

class Windowlist(Item):

	def __init__(self, node=None):
		Item.__init__(self, node, type='windowlist')
		self.editable = True
		self.thisvp = self.node.find('thisvp')
		self.minionly = self.node.find('minionly')

	def get_icon(self):
		iconnode = self.node.find('icon')
		if iconnode is not None:
			return iconnode.text
		else:
			return None

	def get_icon_mode(self):
		iconnode = self.node.find('icon')
		if iconnode is not None:
			if iconnode.attrib.get('mode1') == 'file':
				return iconnode.attrib.get('mode1')
		else:
			return None

	def get_options(self):
		icons = []
		retlist = []

		widget = gtk.CheckButton("Show windows only on current viewport")
		widget.props.active = self.get_thisvp()
		widget.connect('toggled', self.on_thisvp_changed)
		retlist.append(widget)

		widget = gtk.CheckButton("Show only minimized windows")
		widget.props.active = self.get_minionly()
		widget.connect('toggled', self.on_minionly_changed)
		retlist.append(widget)

		iconnode = self.node.find('icon')
		if iconnode is not None:
			icon = iconnode.text
			if iconnode.attrib.get('mode1') == 'file':
				icon_mode = "File path"
			else:
				icon_mode="Normal"
		else:
			icon = ''
			icon_mode="Normal"
		widget = IconSelector(mode=icon_mode,text=icon)

		widget.connect('text-changed', self.on_subnode_changed, 'icon')
		widget.connect('mode-changed', self.on_icon_mode_changed)

		icons.append(widget)

		return icons,retlist

	def on_subnode_changed(self, widget, text, tag):
		subnode = self.node.find(tag)
		if text:
			if subnode is None:
				subnode = etree.SubElement(self.node, tag)
			subnode.text = text
		else:
			if subnode is not None:
				self.node.remove(subnode)

	def on_icon_mode_changed(self, widget, text):
		iconnode = self.node.find('icon')
		if text == "File path":
			if iconnode is None:
				iconnode = etree.SubElement(self.node, 'icon')
			iconnode.attrib['mode1'] = 'file'
		elif 'mode1' in iconnode.attrib:
			del iconnode.attrib['mode1']

	def get_thisvp(self):
		if self.thisvp is not None:
			return self.thisvp.text == 'true'
		return False

	def get_minionly(self):
		if self.minionly is not None:
			return self.minionly.text == 'true'
		return False

	def on_thisvp_changed(self, widget):
		if self.thisvp is None:
			self.thisvp = etree.SubElement(self.node, 'thisvp')
		if widget.props.active:
			text = 'true'
		else:
			text = 'false'
		self.thisvp.text = text

	def on_minionly_changed(self, widget):
		if self.minionly is None:
			self.minionly = etree.SubElement(self.node, 'minionly')
		if widget.props.active:
			text = 'true'
		else:
			text = 'false'
		self.minionly.text = text

	def get_type(self):
		return 'Windows list'

class Viewportlist(Item):

	def __init__(self, node=None): #change to be an attribute of viewportlist
		Item.__init__(self, node, type='viewportlist')
		self.editable = True
		self.wrap = self.node.find('wrap')

	def get_type(self):
		return 'Viewports list'

	def get_icon(self):
		iconnode = self.node.find('icon')
		if iconnode is not None:
			return iconnode.text
		else:
			return None

	def get_icon_mode(self):
		iconnode = self.node.find('icon')
		if iconnode is not None:
			if iconnode.attrib.get('mode1') == 'file':
				return iconnode.attrib.get('mode1')
		else:
			return None


	def get_options(self):
		retlist = []
		icons = []

		widget = gtk.CheckButton("Wrap Viewports")
		widget.props.active = self.get_wrap()
		widget.connect('toggled', self.on_wrap_changed)
		retlist.append(widget)

		iconnode = self.node.find('icon')
		if iconnode is not None:
			icon = iconnode.text
			if iconnode.attrib.get('mode1') == 'file':
				icon_mode = "File path"
			else:
				icon_mode="Normal"
		else:
			icon = ''
			icon_mode="Normal"
		widget = IconSelector(mode=icon_mode,text=icon)

		widget.connect('text-changed', self.on_subnode_changed, 'icon')
		widget.connect('mode-changed', self.on_icon_mode_changed)

		icons.append(widget)

		vpiconnode = self.node.find('vpicon')
		if vpiconnode is not None:
			vpicon = vpiconnode.text
			if vpiconnode.attrib.get('mode1') == 'file':
				vpicon_mode = "File path"
			else:
				vpicon_mode="Normal"
		else:
			vpicon = ''
			vpicon_mode="Normal"
		widget = IconSelector(label_text="Viewport Icon",mode=vpicon_mode,text=vpicon)

		widget.connect('text-changed', self.on_subnode_changed, 'vpicon')
		widget.connect('mode-changed', self.on_vpicon_mode_changed)

		icons.append(widget)

		return icons,retlist

	def get_wrap(self):
		if self.wrap is not None:
			return self.wrap.text == 'true'
		return False

	def on_icon_mode_changed(self, widget, text):
		iconnode = self.node.find('icon')
		if text == "File path":
			if iconnode is None:
				iconnode = etree.SubElement(self.node, 'icon')
			iconnode.attrib['mode1'] = 'file'
		elif 'mode1' in iconnode.attrib:
			del iconnode.attrib['mode1']

	def on_vpicon_mode_changed(self, widget, text):
		iconnode = self.node.find('vpicon')
		if text == "File path":
			if iconnode is None:
				iconnode = etree.SubElement(self.node, 'vpicon')
			iconnode.attrib['mode1'] = 'file'
		elif 'mode1' in iconnode.attrib:
			del iconnode.attrib['mode1']

	def on_subnode_changed(self, widget, text, tag):
		subnode = self.node.find(tag)
		if text:
			if subnode is None:
				subnode = etree.SubElement(self.node, tag)
			subnode.text = text
		else:
			if subnode is not None:
				self.node.remove(subnode)

	def on_wrap_changed(self, widget):
		if self.wrap is None:
			self.wrap = etree.SubElement(self.node, 'wrap')
		if widget.props.active:
			text = 'true'
		else:
			text = 'false'
		self.wrap.text = text

class Documents(Item):

	def __init__(self, node=None): #change to be an attribute of viewportlist
		Item.__init__(self, node, type='documents')
		self.editable = True

	def get_type(self):
		return 'Recent Documents list'

	def get_icon(self):
		iconnode = self.node.find('icon')
		if iconnode is not None:
			return iconnode.text
		else:
			return None

	def get_icon_mode(self):
		iconnode = self.node.find('icon')
		if iconnode is not None:
			if iconnode.attrib.get('mode1') == 'file':
				return iconnode.attrib.get('mode1')
		else:
			return None


	def get_options(self):
		retlist = []
		icons = []
		sgroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)

		label = gtk.Label()
		label.set_alignment(0, 0.5)
		sgroup.add_widget(label)
		label.set_markup('<b>Open method:</b>')
		widget = gtk.Entry()
		commandnode = self.node.find('command')
		if commandnode is not None:
			command = commandnode.text
		else:
			command = ''
		widget.props.text = command
		widget.set_tooltip_text('If you need a more complicated command, type in that command and %f, which will tell compiz-boxmenu where to place the file name.')
		widget.connect('changed', self.on_subnode_changed, 'command')

		hbox = gtk.HBox()
		hbox.pack_start(label)
		hbox.pack_start(widget, True, True)
		retlist.append(hbox)

		iconnode = self.node.find('icon')
		if iconnode is not None:
			icon = iconnode.text
			if iconnode.attrib.get('mode1') == 'file':
				icon_mode = "File path"
			else:
				icon_mode="Normal"
		else:
			icon = ''
			icon_mode="Normal"
		widget = IconSelector(mode=icon_mode,text=icon)

		widget.connect('text-changed', self.on_subnode_changed, 'icon')
		widget.connect('mode-changed', self.on_icon_mode_changed)

		icons.append(widget)

		label = gtk.Label()
		label.set_alignment(0, 0.5)
		sgroup.add_widget(label)
		label.set_markup('<b>Days from today:</b>')
		widget = gtk.Entry()

		agenode = self.node.find('age')
		if agenode is not None:
			age = agenode.text
		else:
			age = ''
		widget.props.text = age
		widget.connect('changed', self.on_subnode_changed, 'age')

		hbox = gtk.HBox()
		hbox.pack_start(label)
		hbox.pack_start(widget, True, True)
		retlist.append(hbox)

		label = gtk.Label()
		label.set_alignment(0, 0.5)
		sgroup.add_widget(label)
		label.set_markup('<b>Items to display:</b>')
		widget = gtk.Entry()

		quantitynode = self.node.find('quantity')
		if quantitynode is not None:
			quantity = quantitynode.text
		else:
			quantity = ''
		widget.props.text = quantity
		widget.connect('changed', self.on_subnode_changed, 'quantity')

		hbox = gtk.HBox()
		hbox.pack_start(label)
		hbox.pack_start(widget, True, True)
		retlist.append(hbox)

		sortnode = self.node.find('sort')

		label = gtk.Label()
		label.set_alignment(0, 0.5)
		sgroup.add_widget(label)
		label.set_markup('<b>Sort mode:</b>')
		widget = gtk.combo_box_new_text()
		widget.append_text('None')
		widget.append_text('Most Used')
		widget.append_text('Least Used')
		if sortnode is None:
			widget.set_active(0)
		elif sortnode.text == 'most used':
			widget.set_active(1)
		elif sortnode.text == 'least used':
			widget.set_active(2)
		else:
			widget.set_active(-1)
		widget.connect('changed', self.on_sort_mode_changed)

		hbox = gtk.HBox()
		hbox.pack_start(label)
		hbox.pack_start(widget, True, True)
		retlist.append(hbox)

		return icons,retlist

	def on_subnode_changed(self, widget, text, tag):
		subnode = self.node.find(tag)
		if text:
			if subnode is None and tag != 'age' and tag != 'quantity':
				subnode = etree.SubElement(self.node, tag)
			elif tag == 'age' or tag == 'quantity':
				if re.search("^\d+$", text): #don't make if it doesn't have text
					if subnode is None:
						subnode = etree.SubElement(self.node, tag)
					subnode.text = text #don't change the numbers if
			else:
				subnode.text = text
		else:
			if subnode is not None:
				self.node.remove(subnode)

	def on_icon_mode_changed(self, widget, text):
		iconnode = self.node.find('icon')
		if text == "File path":
			if iconnode is None:
				iconnode = etree.SubElement(self.node, 'icon')
			iconnode.attrib['mode1'] = 'file'
		elif 'mode1' in iconnode.attrib:
			del iconnode.attrib['mode1']

	def on_sort_mode_changed(self, widget):
		sortnode = self.node.find('sort')
		sorttype = widget.get_active_text()
		if sorttype != 'None':
			print sorttype
			if sortnode is None:
				sortnode = etree.SubElement(self.node, 'sort')
			if sorttype == 'Most Used':
				sortnode.text = 'most used'
			else:
				sortnode.text = 'least used'
		else:
			if sortnode is not None:
				self.node.remove(sortnode)

class Reload(Item):

	def __init__(self, node=None):
		Item.__init__(self, node, type='reload')
		self.editable = True

	def get_icon(self):
		iconnode = self.node.find('icon')
		if iconnode is not None:
			return iconnode.text
		else:
			return None

	def get_icon_mode(self):
		iconnode = self.node.find('icon')
		if iconnode is not None:
			if iconnode.attrib.get('mode1') == 'file':
				return iconnode.attrib.get('mode1')
		else:
			return None

	def get_options(self):
		icons = []
		retlist = []

		iconnode = self.node.find('icon')
		if iconnode is not None:
			icon = iconnode.text
			if iconnode.attrib.get('mode1') == 'file':
				icon_mode = "File path"
			else:
				icon_mode="Normal"
		else:
			icon = ''
			icon_mode="Normal"
		widget = IconSelector(mode=icon_mode,text=icon)

		widget.connect('text-changed', self.on_subnode_changed, 'icon')
		widget.connect('mode-changed', self.on_icon_mode_changed)

		icons.append(widget)

		return icons,retlist


	def on_subnode_changed(self, widget, text, tag):
		subnode = self.node.find(tag)
		if text:
			if subnode is None:
				subnode = etree.SubElement(self.node, tag)
			subnode.text = text
		else:
			if subnode is not None:
				self.node.remove(subnode)

	def on_icon_mode_changed(self, widget, text):
		iconnode = self.node.find('icon')
		if text == "File path":
			if iconnode is None:
				iconnode = etree.SubElement(self.node, 'icon')
			iconnode.attrib['mode1'] = 'file'
		elif 'mode1' in iconnode.attrib:
			del iconnode.attrib['mode1']

	def get_type(self):
		return 'Reload'

class Separator(object):

	def __init__(self, node=None, parent=None):
		self.node = node
		self.editable = True
		if self.node is None:
			self.node = etree.Element('separator')

	def get_name(self):
		name = self.node.attrib.get('name', '---')
		if self.node.attrib.get('mode') == 'exec':
			name = 'exec: %s' %name
		return name

	def get_type(self):
		return 'Separator'

	def get_icon(self):
		return self.node.attrib.get('icon', '')

	def get_icon_mode(self):
		if self.node.attrib.get('mode1') == 'file':
			return self.node.attrib.get('mode1')
		else:
			return None

	def get_options(self):
		icons = []
		retlist = []

		name = self.node.attrib.get('name')
		if name is not None:
			if self.node.attrib.get('mode') == 'exec':
				name_mode="Execute"
			else:
				name_mode="Normal"
		else:
			name = ''
			name_mode="Normal"

		widget = CommandText(mode=name_mode,text=name)
		widget.connect('text-changed', self.on_name_changed)
		widget.connect('mode-changed', self.on_name_mode_changed)

		retlist.append(widget)

		icon = self.node.attrib.get('icon')
		if icon is not None:
			if self.node.attrib.get('mode1') == 'file':
				icon_mode = "File path"
			else:
				icon_mode="Normal"
		else:
			icon = ''
			icon_mode="Normal"

		widget = IconSelector(mode=icon_mode,text=icon)
		widget.connect('text-changed', self.on_icon_changed)
		widget.connect('mode-changed', self.on_icon_mode_changed)

		icons.append(widget)

		return icons,retlist

	def on_name_changed(self, widget, text):
		if text != '':
			self.node.attrib['name'] = text
		else:
			del self.node.attrib['name']

	def on_name_mode_changed(self, widget, text):
		if text == "Execute":
			self.node.attrib['mode'] = 'exec'
		elif 'mode' in self.node.attrib:
			del self.node.attrib['mode']

	def on_icon_changed(self, widget, text):
		if text != '':
			self.node.attrib['icon'] = text
		else:
			del self.node.attrib['icon']

	def on_icon_mode_changed(self, widget, text):
		if text == "File path":
			self.node.attrib['mode1'] = 'file'
		elif 'mode1' in self.node.attrib:
			del self.node.attrib['mode1']

class Menu(object):

	def __init__(self, node=None):
		self.node = node
		self.children = []
		self.editable = True

		if node is None:
			self.node = etree.Element('menu', name='')

		for child in self.node.getchildren():
			try:
				self.children.append(self.make_child(child))
			except KeyError:
				pass


	def make_child(self, child):
		return elements[child.tag](child)

	def get_name(self):
		name = self.node.attrib.get('name', '')
		if self.node.attrib.get('mode') == 'exec':
			name = 'exec: %s' %name
		return name

	def get_icon(self):
		return self.node.attrib.get('icon', '')

	def get_icon_mode(self):
		if self.node.attrib.get('mode1') == 'file':
			return self.node.attrib.get('mode1')
		else:
			return None

	def get_type(self):
		return 'Menu'

	def get_options(self):

		icons = []
		retlist = []

		name = self.node.attrib.get('name')
		if name is not None:
			if self.node.attrib.get('mode') == 'exec':
				name_mode="Execute"
			else:
				name_mode="Normal"
		else:
			name = ''
			name_mode="Normal"

		widget = CommandText(mode=name_mode,text=name)
		widget.connect('text-changed', self.on_name_changed)
		widget.connect('mode-changed', self.on_name_mode_changed)

		retlist.append(widget)

		icon = self.node.attrib.get('icon')
		if icon is not None:
			if self.node.attrib.get('mode1') == 'file':
				icon_mode = "File path"
			else:
				icon_mode="Normal"
		else:
			icon = ''
			icon_mode="Normal"

		widget = IconSelector(mode=icon_mode,text=icon)
		widget.connect('text-changed', self.on_icon_changed)
		widget.connect('mode-changed', self.on_icon_mode_changed)

		icons.append(widget)

		return icons,retlist

	def on_name_changed(self, widget, text):
		self.node.attrib['name'] = text

	def on_name_mode_changed(self, widget, text): #EDIT THIS
		if text == "Execute":
			self.node.attrib['mode'] = 'exec'
		elif 'mode' in self.node.attrib:
			del self.node.attrib['mode']

	def on_icon_changed(self, widget, text):
		if text != '':
			self.node.attrib['icon'] = text
		else:
			del self.node.attrib['icon']

	def on_icon_mode_changed(self, widget, text): #EDIT THIS
		if text == "File path":
			self.node.attrib['mode1'] = 'file'
		elif 'mode1' in self.node.attrib:
			del self.node.attrib['mode1']

itemtypes = {
	'launcher': Launcher,
	'windowlist': Windowlist,
	'viewportlist': Viewportlist,
	'documents': Documents,
	'reload': Reload,
}


def make_item(node):

	itemtype = node.attrib.get('type')
	return itemtypes.get(itemtype, Item)(node)

elements = {'menu': Menu, 'item': make_item, 'separator': Separator}

elementsbyname = {
	'Launcher': Launcher,
	'Windows List': Windowlist,
	'Viewports List': Viewportlist,
	'Recent Documents': Documents,
	'Reload': Reload,
	'Separator': Separator,
	'Menu': Menu,
}


#!/usr/bin/python
import os, gtk, glib
from pyicon_browser import *
import gobject
import subprocess
import shlex
import os

class TabButton(gtk.HBox):
	def __init__(self, text):
			gtk.HBox.__init__(self)
			#http://www.eurion.net/python-snippets/snippet/Notebook%20close%20button.html
			self.label = gtk.Label(text)
			self.pack_start(self.label)

			#get a stock close button image
			close_image = gtk.image_new_from_stock(gtk.STOCK_CLOSE, gtk.ICON_SIZE_MENU)
			image_w, image_h = gtk.icon_size_lookup(gtk.ICON_SIZE_MENU)

			#make the close button
			self.btn = gtk.Button()
			self.btn.set_relief(gtk.RELIEF_NONE)
			self.btn.set_focus_on_click(False)
			self.btn.add(close_image)
			self.pack_start(self.btn, False, False)

			#this reduces the size of the button
			style = gtk.RcStyle()
			style.xthickness = 0
			style.ythickness = 0
			self.btn.modify_style(style)

			self.show_all()

#test code
#from cbutil import *
#import gtk
#d=gtk.Dialog()
#d.vbox.add(CommandText())
#d.run()

class CommandText(gtk.HBox):
	def __init__(self, label_text="Name", mode="Normal", text="", alternate_mode="Execute"):
			gtk.HBox.__init__(self)

			label=gtk.Label(label_text)

			self.entry=gtk.Entry()
			self.entry.props.text=text

			self.button=gtk.Button()
			image=gtk.image_new_from_icon_name("gtk-execute",gtk.ICON_SIZE_LARGE_TOOLBAR)
			self.button.set_image(image)
			#known bug
			self.button.set_tooltip_markup("See the output this command generates")

			self.combobox=gtk.combo_box_new_text()
			self.combobox.append_text("Normal")
			self.combobox.append_text(alternate_mode)
			self.combobox.props.active = mode != "Normal"

			self.add(label)
			self.add(self.entry)
			self.add(self.button)
			self.add(self.combobox)

			self.combobox.connect('changed', self._emit_mode_signal)
			self.entry.connect('changed', self._emit_text_signal)
			self.button.connect('pressed', self._preview_text)

			self.show_all()

			get_mode=self.combobox.get_active_text()
			if get_mode == "Normal":
				self.button.props.sensitive=0
			else:
				self.button.props.sensitive=1

			if alternate_mode != "Execute" or get_mode != "Normal":
				completion = gtk.EntryCompletion()
				self.entry.set_completion(completion)
				completion.set_model(possibility_store)
				completion.set_text_column(0)

	def _emit_mode_signal(self, widget):
		text=self.combobox.get_active_text()
		if text == "Normal":
			self.entry.set_completion(None)
			self.button.props.sensitive=0
		else:
			completion = gtk.EntryCompletion()
			self.entry.set_completion(completion)
			completion.set_model(possibility_store)
			completion.set_text_column(0)
			self.button.props.sensitive=1
		self.emit('mode-changed', text)

	def _emit_text_signal(self, widget):
		self.emit('text-changed', widget.props.text)

	def _preview_text(self, widget):
		print "Generating preview, please wait..."
		buffer=gtk.TextBuffer()
		buffer_errors=gtk.TextBuffer()
		full_text=' '.join(['/usr/bin/env',os.path.expanduser(self.entry.props.text)])
		cmd=subprocess.Popen(shlex.split(full_text),stdout=subprocess.PIPE, stderr=subprocess.PIPE)
		text,errors=cmd.communicate()
		if text != None:
			buffer.set_text(text)
		else:
			buffer.set_text("")

		if errors != None:
			buffer_errors.set_text(errors)
		else:
			buffer_errors.set_text("")

		dialog=gtk.Dialog(title="Preview of %s" %(self.entry.props.text), \
			buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))

		tabs=gtk.Notebook()
		tabs.set_scrollable(True)

		scrolled=gtk.ScrolledWindow()
		scrolled.add(gtk.TextView(buffer))

		scrolled_errors=gtk.ScrolledWindow()
		scrolled_errors.add(gtk.TextView(buffer_errors))

		tabs.append_page(scrolled, gtk.Label("Output"))
		tabs.append_page(scrolled_errors, gtk.Label("Errors"))

		dialog.vbox.add(tabs)
		dialog.show_all()
		dialog.run()
		dialog.destroy()

#test code
#from cbutil import *
#import gtk
#d=gtk.Dialog()
#d.vbox.add(IconSelector())
#d.run()

class IconSelector(gtk.HBox):
	def __init__(self, label_text="Icon", mode="Normal", text=""):
			gtk.HBox.__init__(self)

			label=gtk.Label(label_text)
			self.text=text

			self.combobox=gtk.combo_box_new_text()
			self.combobox.append_text("Normal")
			self.combobox.append_text("File path")
			self.combobox.props.active = mode != "Normal"
			self.button=gtk.Button()
			self.image=gtk.Image()

			self.add(label)
			self.add(self.button)
			self.add(self.combobox)
			self.button.set_image(self.image)

			self.combobox.connect('changed', self._emit_mode_signal)
			self.button.connect('pressed', self._emit_text_signal)

			if mode == "File path":
				size=gtk.icon_size_lookup(gtk.ICON_SIZE_LARGE_TOOLBAR)[0]
				try:
					pixbuf=gtk.gdk.pixbuf_new_from_file_at_size(os.path.expanduser(self.text), size, size)
					self.image.set_from_pixbuf(pixbuf)
				except glib.GError:
					self.image.set_from_pixbuf(None)
					print "Couldn't set icon from file: %s" %(self.text)
			else:
				self.image.set_from_icon_name(self.text,gtk.ICON_SIZE_LARGE_TOOLBAR)

			self.show_all()

	def _change_image(self, mode):
		if mode == "File path":
			size=gtk.icon_size_lookup(gtk.ICON_SIZE_LARGE_TOOLBAR)[0]
			try:
				pixbuf=gtk.gdk.pixbuf_new_from_file_at_size(os.path.expanduser(self.text), size, size)
				self.image.set_from_pixbuf(pixbuf)
			except glib.GError:
				self.image.set_from_pixbuf(None)
				print "Couldn't set icon from file: %s" %(self.text)
		else:
			self.image.set_from_icon_name(self.text,gtk.ICON_SIZE_LARGE_TOOLBAR)
		self.emit('image-changed', mode)

	def _emit_mode_signal(self, widget):
		text=self.combobox.get_active_text()
		self._change_image(text)
		self.emit('mode-changed', text)

	def _emit_text_signal(self, widget):
		text=""
		if self.combobox.get_active_text() == "Normal":
			dialog=IcoBrowse()
			dialog.set_defaults(self.text)
			response = dialog.run()
			if response == gtk.RESPONSE_ACCEPT:
				text=dialog.get_icon_name(None)
			dialog.destroy()
		else:
			btns=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_ACCEPT)
			dialog=gtk.FileChooserDialog(title="Select Icon", buttons=btns)
			dialog.set_filename(text)
			filter = gtk.FileFilter()
			filter.set_name("Images")
			filter.add_mime_type("image/png")
			filter.add_mime_type("image/jpeg")
			filter.add_mime_type("image/gif")
			filter.add_pattern("*.png")
			filter.add_pattern("*.jpg")
			filter.add_pattern("*.gif")
			filter.add_pattern("*.tif")
			filter.add_pattern("*.xpm")
			dialog.add_filter(filter)
			response = dialog.run()
			if response == gtk.RESPONSE_ACCEPT:
				text=dialog.get_filename()
			dialog.destroy()
		if text != self.text and (text != "" and text != None):
			self.text = text
			self._change_image(self.combobox.get_active_text())
			self.emit('text-changed', text)

def set_up():
	gobject.type_register(CommandText)
	gobject.type_register(IconSelector)

	gobject.signal_new("text-changed", CommandText, gobject.SIGNAL_RUN_FIRST,  gobject.TYPE_NONE, (gobject.TYPE_STRING,))
	gobject.signal_new("mode-changed", CommandText, gobject.SIGNAL_RUN_FIRST,  gobject.TYPE_NONE, (gobject.TYPE_STRING,))

	gobject.signal_new("image-changed", IconSelector, gobject.SIGNAL_RUN_FIRST,  gobject.TYPE_NONE, (gobject.TYPE_STRING,))
	gobject.signal_new("text-changed", IconSelector, gobject.SIGNAL_RUN_FIRST,  gobject.TYPE_NONE, (gobject.TYPE_STRING,))
	gobject.signal_new("mode-changed", IconSelector, gobject.SIGNAL_RUN_FIRST,  gobject.TYPE_NONE, (gobject.TYPE_STRING,))

def completion_setup():
	print "Setting up command auto completion for best experience"
	for i in os.path.expandvars("$PATH").split(":"):
		if os.path.exists(i):
			print "Looking in %s" %i
			for j in os.listdir(i):
				path="%s/%s" %(i,j)
				if not os.path.isdir(path) and \
				os.access(path, os.X_OK):
					possibility_store.append([j])

possibility_store = gtk.ListStore(str)
set_up()
completion_setup()